@@ -335,7 +335,11 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 		}
 	}
 	*--p = '\0';
-	if (strcmp(reason, "TEST") == 0) {
+	if (strcmp(reason, "TEST") == 0 ||
+	    strcmp(reason, "PREINIT") == 0 ||
+	    strcmp(reason, "CARRIER") == 0 ||
+	    strcmp(reason, "UNKNOWN") == 0)
+	{
 		env[9] = strdup("if_up=false");
 		env[10] = strdup("if_down=false");
 	} else if (1 == 2 /* appease ifdefs */
